@@ -64,13 +64,13 @@ def files(tmp_path_factory):
 def _randbytes(n):
     # Backported from Python 3.9
     if not n:
-        return b''
-    return random.getrandbits(n * 8).to_bytes(n, 'little')
+        return b""
+    return random.getrandbits(n * 8).to_bytes(n, "little")
 
 def _get_zip(data):
     if not isinstance(data, io.IOBase):
         if not isinstance(data, (bytes, bytearray)):
-            data = b''.join(data)
+            data = b"".join(data)
         data = io.BytesIO(data)
 
     return zipfile.ZipFile(data)
@@ -89,8 +89,6 @@ def _assert_equal_zips(z1, z2):
     assert z1.comment == z2.comment
     z1i = sorted(z1.infolist(), key=lambda x: x.filename)
     z2i = sorted(z2.infolist(), key=lambda x: x.filename)
-    print(z1i)
-    print(z2i)
     assert len(z1i) == len(z2i)
     for x1, x2, in zip(z1i, z2i):
         assert x1.filename == x2.filename
@@ -137,6 +135,62 @@ def test_zipstream_compression(caplog, files, ct):
         _verify_zip_contains(zf, f)
 
 
+@pytest.mark.parametrize("ct", [
+    zipfile.ZIP_STORED,
+    zipfile.ZIP_LZMA,
+    zipfile.ZIP_DEFLATED,
+    zipfile.ZIP_BZIP2
+])
+@pytest.mark.parametrize("cl", [None, 2])
+def test_mixed_compression_and_getinfo(ct, cl):
+    """Test that files are compressed using the correct method and level and
+    that the information from get_info accurately reflects this"""
+
+    # Make the test just test compression types on 3.6 and below
+    cl, TEST_CL = (None, None) if PY36 else (cl, 5)
+
+    zs = ZipStream(compress_type=ct, compress_level=cl)
+    zs.add(b"0", arcname="0")
+    zs.add(b"1", arcname="1", compress_type=zipfile.ZIP_STORED)
+    zs.add(b"1c", arcname="1c", compress_type=zipfile.ZIP_STORED, compress_level=TEST_CL)
+    zs.add(b"2", arcname="2", compress_type=zipfile.ZIP_LZMA)
+    zs.add(b"2c", arcname="2c", compress_type=zipfile.ZIP_LZMA, compress_level=TEST_CL)
+    zs.add(b"3", arcname="3", compress_type=zipfile.ZIP_DEFLATED)
+    zs.add(b"3c", arcname="3c", compress_type=zipfile.ZIP_DEFLATED, compress_level=TEST_CL)
+    zs.add(b"4", arcname="4", compress_type=zipfile.ZIP_BZIP2)
+    zs.add(b"4c", arcname="4c", compress_type=zipfile.ZIP_BZIP2, compress_level=TEST_CL)
+
+    zf = _get_zip(zs)
+    zinfos = zf.infolist()
+    fullinfos = zs.get_info()
+    assert len(zinfos) == len(fullinfos) == 9
+
+    def assert_zinfo(idx, name, compress_type, compress_level):
+        zi = zinfos[idx]
+        fi = fullinfos[idx]
+        assert fi["name"] == zi.filename == name
+
+        assert fi["compress_type"] == zi.compress_type == compress_type
+        assert fi["compress_level"] == compress_level
+        assert fi["compressed_size"] == zi.compress_size
+
+        assert zf.read(zi) == name.encode("utf-8")
+
+    # no compression level for stored and lzma
+    if ct in (zipfile.ZIP_STORED, zipfile.ZIP_LZMA):
+        assert_zinfo(0, "0", ct, None)
+    else:
+        assert_zinfo(0, "0", ct, cl)
+    assert_zinfo(1, "1", zipfile.ZIP_STORED, None)
+    assert_zinfo(2, "1c", zipfile.ZIP_STORED, None)
+    assert_zinfo(3, "2", zipfile.ZIP_LZMA, None)
+    assert_zinfo(4, "2c", zipfile.ZIP_LZMA, None)
+    assert_zinfo(5, "3", zipfile.ZIP_DEFLATED, cl)
+    assert_zinfo(6, "3c", zipfile.ZIP_DEFLATED, TEST_CL)
+    assert_zinfo(7, "4", zipfile.ZIP_BZIP2, cl)
+    assert_zinfo(8, "4c", zipfile.ZIP_BZIP2, TEST_CL)
+
+
 @pytest.mark.parametrize("zip64", [False, True])
 def test_zipstream_normal_paths(monkeypatch, files, zip64):
     """Test adding paths and iterating"""
@@ -175,7 +229,7 @@ def test_zipstream_all_then_footer_paths(files):
 def test_per_file_iteration(files):
     """Test iterating per-file while adding files works"""
     zs = ZipStream()
-    data = b''
+    data = b""
     assert zs.num_streamed() == 0
     assert zs.num_queued() == 0
     assert zs.is_empty()
@@ -187,12 +241,12 @@ def test_per_file_iteration(files):
     assert zs.num_queued() == 3
     assert not zs.is_empty()
     assert zs
-    data += b''.join(zs.file())
+    data += b"".join(zs.file())
     assert zs.num_streamed() == 1
     assert zs.num_queued() == 2
     assert not zs.is_empty()
     assert zs
-    data += b''.join(zs.all_files())
+    data += b"".join(zs.all_files())
     assert zs.num_streamed() == 3
     assert zs.num_queued() == 0
     assert not zs.is_empty()
@@ -202,7 +256,7 @@ def test_per_file_iteration(files):
     assert zs.num_queued() == 1
     assert not zs.is_empty()
     assert zs
-    data += b''.join(zs.finalize())
+    data += b"".join(zs.finalize())
     assert zs.num_streamed() == 4
     assert zs.num_queued() == 0
     assert not zs.is_empty()
@@ -344,6 +398,9 @@ def test_adding_windows_paths(monkeypatch):
     zs.add("", "a/b\\")
     zs.add("", "a\\b/")
     zs.add("", "a\\b\\")
+    zs.mkdir("a/b\\")
+    zs.mkdir("a\\b/")
+    zs.mkdir("a\\b\\")
 
     predicted = len(zs)
     actual = len(bytes(zs))
@@ -351,7 +408,7 @@ def test_adding_windows_paths(monkeypatch):
     assert predicted == actual
 
     info = zs.get_info()
-    assert len(info) == 3
+    assert len(info) == 6
     for x in info:
         assert x["name"] == "a/b/"
 
@@ -401,7 +458,7 @@ def test_creating_dirs_with_data():
     """Test creating directories works except when adding data to them"""
     zs = ZipStream()
     zs.add(None, "folder0/")
-    zs.add(b'', "folder1/")
+    zs.add(b"", "folder1/")
     zs.add("", "folder2/")
     with pytest.raises(ValueError):
         zs.add("data", "folder3/")
@@ -415,6 +472,35 @@ def test_creating_dirs_with_data():
         assert zinfos[i].compress_size == 0
 
 
+def test_mkdir():
+    zs = ZipStream(sized=True)
+
+    with pytest.raises(ValueError, match="A valid arcname .* is required"):
+        zs.mkdir("")
+
+    PATHS = (
+        "folder0",  # no trailing slash
+        "folder1",
+        "folder1/sub",  # into existing dir
+        "folder2/sub/sub/sub/",  # new path
+    )
+
+    for p in PATHS:
+        zs.mkdir(p)
+
+    data = bytes(zs)
+    assert len(data) == len(zs)
+
+    zinfos = _get_zip(data).infolist()
+    assert len(zinfos) == 4
+
+    for i, p in enumerate(x.rstrip("/") + "/" for x in PATHS):
+        assert zinfos[i].filename == p
+        assert zinfos[i].is_dir()
+        assert zinfos[i].file_size == 0
+        assert zinfos[i].compress_size == 0
+
+
 def test_directly_adding_empty_dir(tmpdir):
     """Test adding an empty directory"""
     t = tmpdir.mkdir("empty")
@@ -423,7 +509,7 @@ def test_directly_adding_empty_dir(tmpdir):
     data = bytes(zs)
     assert len(data) == len(zs)
 
-    zinfos = sorted(_get_zip(data).infolist(), key=lambda x: x.filename)
+    zinfos = _get_zip(data).infolist()
     assert len(zinfos) == 1
     assert zinfos[0].filename == "empty/"
     assert zinfos[0].is_dir()
@@ -435,7 +521,7 @@ def test_adding_local_dir(tmpdir):
     """Test adding files/folder from the local directory doesn't include the directory name of ."""
     t = tmpdir.mkdir("top")
     t.mkdir("empty")
-    t.mkdir("not_empty").join("file.txt").write(b'x')
+    t.mkdir("not_empty").join("file.txt").write(b"x")
 
     os.chdir(str(t))
 
@@ -460,7 +546,7 @@ def test_empty_folders_preserved_recursive(tmpdir):
     """Test that recursively adding a directory preserves empty files and folders in it"""
     t = tmpdir.mkdir("top")
     t.mkdir("empty")
-    t.join("file.txt").write(b'')
+    t.join("file.txt").write(b"")
 
     zs = ZipStream.from_path(t)
 
@@ -485,7 +571,7 @@ def test_recursion_disable(tmpdir):
     """Test that recursion can be disabled to just add a single (empty) folder"""
     t = tmpdir.mkdir("top")
     t.mkdir("empty")
-    t.join("file.txt").write(b'')
+    t.join("file.txt").write(b"")
 
     zinfos = _get_zip(ZipStream.from_path(t, recurse=False)).infolist()
 
@@ -496,11 +582,98 @@ def test_recursion_disable(tmpdir):
     assert zinfos[0].compress_size == 0
 
 
+@pytest.mark.parametrize("preserve_empty", [True, False])
+@pytest.mark.parametrize("followlinks", [True, False])
+def test_zipstream_walk(tmpdir, preserve_empty, followlinks):
+    """Test the zipstream's walk walks the walk it talks"""
+
+    # Create directory structure
+    t = tmpdir.mkdir("top")
+    t.mkdir("empty")
+    t.join("empty.txt").write("")
+    t.join("test.txt").write("test")
+    f = t.mkdir("filled")
+    f.join("file.bin").write_binary(b"\x00\x01\xFF")
+
+    # Can't make symlinks on some platforms
+    symlink_support = hasattr(t, "mksymlinkto")
+    if symlink_support:
+        f.join("file2.bin").mksymlinkto("file.bin")
+
+        o = tmpdir.mkdir("othertop")
+        o.join("other.txt").write("other")
+        f.join("notevil").mksymlinkto("../../othertop")
+        f.join("evil").mksymlinkto("../")
+
+    def walk(path):
+        yield from zipstream.walk(
+            path,
+            preserve_empty=preserve_empty,
+            followlinks=followlinks
+        )
+
+    zs = ZipStream.from_path(t, recurse=walk)
+    data = bytes(zs)
+    assert len(data) == len(zs)
+
+    contents = {
+        x.filename: x.is_dir()
+        for x in _get_zip(data).infolist()
+    }
+
+    for name, is_dir in contents.items():
+        assert (name == "top/empty/") == is_dir
+
+    names = set(contents)
+    assert "top/empty.txt" in names
+    assert symlink_support == ("top/filled/file2.bin" in names)
+    assert "top/filled/file.bin" in names
+    assert "top/test.txt" in names
+    assert preserve_empty == ("top/empty/" in names)
+    assert (symlink_support and followlinks) == ("top/filled/notevil/other.txt" in names)
+
+
+def test_custom_walk(tmpdir):
+    """Test that custom walk functions are supported"""
+
+    t = tmpdir.mkdir("top")
+    t.join("1.txt").write("1")
+    t.join("2.txt").write("22")
+    t.join("3.bin").write("333")
+    t.join("4.txt").write("4444")
+    t.join("5.txt").write("55555")
+    t.mkdir("empty")
+
+    def custom_walk(path):
+        for dirpath, dirnames, files in os.walk(path):
+            for f in files:
+                if f.endswith(".bin"):
+                    continue
+                fullpath = os.path.join(dirpath, f)
+                if os.path.getsize(fullpath) > 4:
+                    continue
+                yield fullpath
+
+            if not files and not dirnames:
+                yield dirpath  # no trailing pathsep
+
+    zs = ZipStream.from_path(t, recurse=custom_walk)
+    data = bytes(zs)
+    assert len(data) == len(zs)
+    zf = _get_zip(data)
+    assert zf.getinfo("top/empty/").is_dir()
+
+    names = sorted(x.filename for x in zf.infolist())
+    assert len(names) == 4
+    assert "top/3.bin" not in names
+    assert "top/5.txt" not in names
+
+
 @pytest.mark.parametrize("data", [
     "this is a string",
-    b'these are some bytes',
-    bytearray(b'this is a bytearray'),
-    [b'a', b'list', b'of', b'bytes'],
+    b"these are some bytes",
+    bytearray(b"this is a bytearray"),
+    [b"a", b"list", b"of", b"bytes"],
     _gen_rand()
 ])
 @pytest.mark.parametrize("ct", [
@@ -523,11 +696,15 @@ def test_adding_data(caplog, data, ct):
         # tee the iterator and get the contents from it so they can be checked
         # against what's put into the stream
         tostore, data = itertools.tee(data)
-        data = b''.join(data)
+        data = b"".join(data)
 
     # Test arcname is required
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="A valid arcname .* is required"):
+        zs.add(tostore, None)
+    with pytest.raises(ValueError, match="A valid arcname .* is required"):
         zs.add(tostore, "")
+    with pytest.raises(ValueError, match="Can't store .* as a directory"):
+        zs.add(tostore, "directory/")
 
     zs.add(tostore, "data.bin")
     zf = _get_zip(zs)
@@ -550,6 +727,24 @@ def test_adding_invalid_data(data):
         zs.add(data, "test")
 
 
+@pytest.mark.parametrize("name", ["file", "directory/"])
+def test_adding_broken_symlinks(tmpdir, name):
+    """Test that adding a broken symlink will raise a FileNotFoundError"""
+
+    t = tmpdir.mkdir("top")
+    if not hasattr(t, "mksymlinkto"):
+        pytest.skip("mksymlinkto not supported - can't test broken links")
+
+    l = t.join(name)
+    l.mksymlinkto("nothing")
+
+    zs = ZipStream()
+    with pytest.raises(FileNotFoundError):
+        zs.add_path(l)
+    with pytest.raises(FileNotFoundError):
+        zs.add_path(t)
+
+
 @pytest.mark.parametrize("data", [
     ["strs", "cannot", "be", "added"],
     range(10),
@@ -603,7 +798,7 @@ def test_directory_links_without_infinite_recursion(tmpdir):
 
 
 def test_adding_missing_path(tmpdir):
-    with pytest.raises(ValueError):
+    with pytest.raises(FileNotFoundError):
         ZipStream.from_path(tmpdir.join("doesntexist"))
 
 
@@ -620,6 +815,43 @@ def test_adding_deleted_path(tmpdir):
         bytes(zs)
 
 
+def test_adding_empty_name(tmpdir, monkeypatch):
+    """Test that when trying to discover an arcname for a path empty names raise an error"""
+    monkeypatch.setattr(os.path, "basename", lambda _: "")
+    zs = ZipStream(sized=True)
+    with pytest.raises(ValueError, match="No arcname for path"):
+        zs.add_path(tmpdir)
+    with pytest.raises(ValueError, match="No arcname for path"):
+        zs.add_path(tmpdir, arcname="")
+
+
+def test_adding_null_byte_name():
+    zs = ZipStream(sized=True)
+
+    zs.add(b"data", "file.txt\x00and more")
+    with pytest.raises(ValueError, match="A valid arcname .* is required"):
+        zs.add(b"otherdata", "\x00nofilename")
+
+    zs.mkdir("directory\x00and more")
+    with pytest.raises(ValueError, match="A valid arcname .* is required"):
+        zs.mkdir("\x00nodirectory")
+
+    expected_len = len(zs)
+    data = bytes(zs)
+    assert len(data) == expected_len
+
+    zinfos = sorted(_get_zip(data).infolist(), key=lambda x: x.filename)
+    assert len(zinfos) == 2
+
+    assert zinfos[0].filename == "directory/"
+    assert zinfos[0].file_size == 0
+    assert zinfos[0].is_dir()
+
+    assert zinfos[1].filename == "file.txt"
+    assert zinfos[1].file_size == 4
+    assert not zinfos[1].is_dir()
+
+
 @pytest.mark.parametrize("data", [
     ("short text", "then longer text"),
     ("first longer text", "then shorter"),
@@ -654,7 +886,7 @@ def test_adding_iterator_invalid_size(caplog, sized, size):
     """Make sure errors are raised when generating if an incorrect iterator size was provided"""
     caplog.set_level(logging.WARNING)
 
-    l_rand = len(b''.join(_gen_rand()))
+    l_rand = len(b"".join(_gen_rand()))
 
     zs = ZipStream(sized=sized)
     zs.add(_gen_rand(), "rand.txt", size=size)
@@ -663,10 +895,55 @@ def test_adding_iterator_invalid_size(caplog, sized, size):
     if sized and size is not None:
         with pytest.raises(RuntimeError, match="Error adding 'rand.txt' to sized ZipStream"):
             bytes(zs)
-        assert "Size mismatch when adding data for 'rand.txt'" in caplog.text
     else:
         bytes(zs)
-        assert "Size mismatch" not in caplog.text
+
+    # only emits a warning when size was specfied
+    assert (size is not None) == ("Size mismatch when adding data for 'rand.txt'" in caplog.text)
+
+
+def test_adding_iterator_invalid_size_underestimate_zip64(monkeypatch):
+    """Make sure that if an iterator size that doesn't require zip64 is given
+    and the data from it does, a RuntimeError is raised"""
+    monkeypatch.setattr(zipfile, "ZIP64_LIMIT", 100)
+    monkeypatch.setattr(zipstream.ng, "ZIP64_LIMIT", 100)
+
+    zs = ZipStream(sized=False)
+    zs.add(_gen_rand(), "rand.txt", size=1)
+
+    with pytest.raises(RuntimeError, match="Adding file 'rand.txt' unexpectedly required using Zip64 extensions"):
+        bytes(zs)
+
+
+def test_underestimate_zip64_via_compression(monkeypatch):
+    """Test that underestimating compresed size raises a RuntimeError"""
+
+    # compressed data can be bigger than uncompressed data. This generally only
+    # happens with small amounts of data but can with large amounts of data as
+    # well. This is less of a test and more of a demonstration of why the
+    # estimation factor is needed
+    monkeypatch.setattr(zipfile, "ZIP64_LIMIT", 10)
+    monkeypatch.setattr(zipstream.ng, "ZIP64_LIMIT", 10)
+
+    ZEROS = "0" * 10
+    NUMS = "0123456789"
+
+    def _make_zip(content):
+        zs = ZipStream(compress_type=zipfile.ZIP_DEFLATED)
+        zs.add(content, "file.txt")
+        bytes(zs)
+
+    assert len(ZEROS) == len(NUMS)
+
+    _make_zip(ZEROS)
+    _make_zip(NUMS)
+
+    # remove overestimation
+    monkeypatch.setattr(zipstream.ng, "ZIP64_ESTIMATE_FACTOR", 1)
+
+    _make_zip(ZEROS)
+    with pytest.raises(RuntimeError, match="Adding file 'file.txt' unexpectedly required using Zip64 extensions"):
+        _make_zip(NUMS)
 
 
 def test_adding_comment(caplog):
@@ -689,7 +966,7 @@ def test_adding_comment(caplog):
     zs.comment = bytearray(b"test")
     assert zs.comment == "test".encode("utf-8")
 
-    c = b'this is a test with a big comment' + bytes(zipfile.ZIP_MAX_COMMENT)
+    c = b"this is a test with a big comment" + bytes(zipfile.ZIP_MAX_COMMENT)
     l = len(c)
     assert "too long" not in caplog.text
     zs.comment = c
@@ -718,17 +995,17 @@ def test_finalizing():
     assert zs._final
 
     # Doesn't finalize if just asking for file data
-    data = b''
+    data = b""
     zs = ZipStream()
     zs.add("", "a")
-    data += b''.join(zs.file())
+    data += b"".join(zs.file())
     assert not zs._final
-    data += b''.join(zs.file())
+    data += b"".join(zs.file())
     assert not zs._final
     zs.add("", "b")
-    data += b''.join(zs.all_files())
+    data += b"".join(zs.all_files())
     assert not zs._final
-    data += b''.join(zs.footer())
+    data += b"".join(zs.footer())
     assert zs._final
     _get_zip(data)
 
@@ -852,7 +1129,7 @@ def test_get_info(monkeypatch):
     zs.add(b"test", "text2.txt")
     assert zs.num_queued() == 3
     assert len(zs.get_info()) == 0
-    data += b''.join(zs.all_files())
+    data += b"".join(zs.all_files())
     assert zs.num_queued() == 0
     info = zs.get_info()
     assert len(info) == 3
@@ -934,7 +1211,7 @@ def test_readme_stdlib_comparison(tmpdir):
                 yield os.path.join(dirpath, f)
 
     def read_file(path):
-        with open(path, 'rb') as fp:
+        with open(path, "rb") as fp:
             while True:
                 buf = fp.read(1024 * 64)
                 if not buf:
@@ -943,7 +1220,7 @@ def test_readme_stdlib_comparison(tmpdir):
 
     def generate_zipstream(path):
         stream = Stream()
-        with ZipFile(stream, mode='w') as zf:
+        with ZipFile(stream, mode="w") as zf:
             toplevel = os.path.basename(os.path.normpath(path))
             for f in iter_files(path):
                 # Use the basename of the path to set the arcname
@@ -951,7 +1228,7 @@ def test_readme_stdlib_comparison(tmpdir):
                 zinfo = ZipInfo.from_file(f, arcname)
 
                 # Write data to the zip file then yield the stream content
-                with zf.open(zinfo, mode='w') as fp:
+                with zf.open(zinfo, mode="w") as fp:
                     if zinfo.is_dir():
                         continue
                     for buf in read_file(f):
@@ -962,21 +1239,21 @@ def test_readme_stdlib_comparison(tmpdir):
     # Create directory structure
     t = tmpdir.mkdir("top")
     t.mkdir("empty")
-    t.join("empty.txt").write('')
-    t.join("test.txt").write('test')
+    t.join("empty.txt").write("")
+    t.join("test.txt").write("test")
     f = t.mkdir("filled")
-    f.join("file.bin").write_binary(b'\x00\x01\xFF')
+    f.join("file.bin").write_binary(b"\x00\x01\xFF")
 
     # Can't make symlinks on some platforms
     if hasattr(t, "mksymlinkto"):
-        f.join("file2.bin").mksymlinkto('file.bin')
+        f.join("file2.bin").mksymlinkto("file.bin")
 
         o = tmpdir.mkdir("othertop")
-        o.join("other.txt").write('other')
-        f.join("notevil").mksymlinkto('../../othertop')
+        o.join("other.txt").write("other")
+        f.join("notevil").mksymlinkto("../../othertop")
 
         # TODO: make stdlib version handle infinte loops
-        #f.join("evil").mksymlinkto('../')
+        #f.join("evil").mksymlinkto("../")
 
     _assert_equal_zips(
         _get_zip(generate_zipstream(t)),
@@ -1014,21 +1291,28 @@ def test_unsized_zipstream_len_typeerror():
 
 @pytest.mark.parametrize("zip64", [False, True])
 @pytest.mark.parametrize("sized", [False, True])
-def test_proper_zip64_min_version(monkeypatch, zip64, sized):
-    """Ensure that the min version is set properly"""
+@pytest.mark.parametrize("known_size", [False, True])
+def test_proper_zip64_min_version(monkeypatch, zip64, sized, known_size):
+    """Ensure that the zip64 extensions are used when needed and the min
+    version is set accordingly
+    """
     if zip64:
         monkeypatch.setattr(zipfile, "ZIP64_LIMIT", 100)
         monkeypatch.setattr(zipstream.ng, "ZIP64_LIMIT", 100)
 
     zs = ZipStream(sized=sized)
-    zs.add(_gen_rand(), "rand.txt")
+    size = None if not known_size else len(b"".join(_gen_rand()))
+
+    zs.add(_gen_rand(), "rand.txt", size=size)
 
     zi = _get_zip(zs).infolist()[0]
-    # A sized zipstream will pull the entire iterator into memory and realize
-    # that zip64 is not required.
+    # A sized zipstream will always pull the entire iterator into memory and
+    # realize that zip64 is not required. Likewise if the size of the iterator
+    # is known.
     # An unsized zipstream will not pull it into memory and be forced to use
-    # zip64 extensions since the size could be too big
-    expected_version = (zipfile.DEFAULT_VERSION if sized and not zip64 else zipfile.ZIP64_VERSION)
+    # zip64 extensions if the size of the iterator is unknown since the size
+    # could be too big.
+    expected_version = (zipfile.DEFAULT_VERSION if not zip64 and (sized or known_size) else zipfile.ZIP64_VERSION)
 
     assert zi.create_version == expected_version
     assert zi.extract_version == expected_version
@@ -1046,7 +1330,7 @@ def test_adding_iterator_sized(sized, known_size):
         yield from _gen_rand()
         read = True
 
-    l_rand = len(b''.join(_gen_rand()))
+    l_rand = len(b"".join(_gen_rand()))
 
     zs = ZipStream(sized=sized)
     zs.add(_my_iter(), "rand.txt", size=None if not known_size else l_rand)
@@ -1153,7 +1437,7 @@ def test_sized_zipstream_size_while_adding(monkeypatch, files, zip64):
         _verify_zip_contains(zf, f)
 
 
-# Warning: skippped because it creates a 4GB+ temp file
+# Warning: skipped because it creates a 4GB+ temp file
 @pytest.mark.skip
 def test_zip64_real(tmpdir):
     """Test compressing a large file using Zip64 extensions works"""
@@ -1169,7 +1453,7 @@ def test_zip64_real(tmpdir):
     smalldata = _randbytes(datasize)
 
     # Write large file
-    with open(str(large_file), 'wb') as fp:
+    with open(str(large_file), "wb") as fp:
         fp.write(startdata)
         fp.seek(zipfile.ZIP64_LIMIT, io.SEEK_CUR)
         fp.write(enddata)
@@ -1182,19 +1466,19 @@ def test_zip64_real(tmpdir):
     calculated = len(zs)
 
     l = 0
-    with open(str(zip_file), 'wb') as fp:
+    with open(str(zip_file), "wb") as fp:
         for chunk in zs:
             l += len(chunk)
             fp.write(chunk)
 
     assert l == calculated
 
-    with zipfile.ZipFile(str(zip_file), 'r') as zf:
+    with zipfile.ZipFile(str(zip_file), "r") as zf:
         zinfos = zf.infolist()
         assert len(zinfos) == 2
 
         # Read the data and make sure it's the same as what was put in
-        with zf.open("large.bin", 'r') as fp:
+        with zf.open("large.bin", "r") as fp:
             assert startdata == fp.read(datasize)
             if PY36:
                 # No support for seeking <3.7 - read it all in 128MB chunks
@@ -1207,5 +1491,5 @@ def test_zip64_real(tmpdir):
                 fp.seek(zipfile.ZIP64_LIMIT, io.SEEK_CUR)
             assert enddata == fp.read()
 
-        with zf.open("small.bin", 'r') as fp:
+        with zf.open("small.bin", "r") as fp:
             assert smalldata == fp.read()
